@@ -169,22 +169,13 @@ where
 /// #[derive(Clone)]
 /// struct ListError(&'static str);
 ///
-/// #[derive(Clone)]
+/// #[derive(Clone, PartialEq)]
 /// enum ListSpace {
 ///     Num(i32),
 ///     NumList(Vec<i32>),
 ///     Func(LiftedFunction<ListSpace, ListsEvaluator>),
 /// }
 /// use ListSpace::*;
-/// impl PartialEq for ListSpace {
-///     fn eq(&self, other: &Self) -> bool {
-///         match (self, other) {
-///             (&Num(x), &Num(y)) => x == y,
-///             (&NumList(ref xs), &NumList(ref ys)) => xs == ys,
-///             _ => false,
-///         }
-///     }
-/// }
 ///
 /// #[derive(Copy, Clone)]
 /// struct ListsEvaluator;
@@ -300,23 +291,13 @@ pub trait Evaluator: Sized + Sync {
 /// #[derive(Clone, Debug, PartialEq)]
 /// struct ListError(&'static str);
 ///
-/// #[derive(Clone, Debug)]
+/// #[derive(Clone, Debug, PartialEq)]
 /// enum ListSpace {
 ///     Bool(bool),
 ///     Num(i32),
 ///     List(Vec<i32>),
 /// }
 /// use ListSpace::*;
-/// impl PartialEq for ListSpace {
-///     fn eq(&self, other: &Self) -> bool {
-///         match (self, other) {
-///             (&Bool(x), &Bool(y)) => x == y,
-///             (&Num(x), &Num(y)) => x == y,
-///             (&List(ref xs), &List(ref ys)) => xs == ys,
-///             _ => false,
-///         }
-///     }
-/// }
 ///
 /// #[derive(Copy, Clone)]
 /// struct ListsEvaluator;
@@ -473,6 +454,21 @@ where
         LiftedFunction(self.0.clone(), self.1.clone(), self.2.clone())
     }
 }
+impl<V, E> PartialEq for LiftedFunction<V, E>
+where
+    E: Evaluator<Space = V>,
+    V: Clone + PartialEq + Send + Sync,
+{
+    fn eq(&self, other: &LiftedFunction<V, E>) -> bool {
+        self.0 == other.0 && self.2 == other.2
+    }
+}
+impl<V, E> Eq for LiftedFunction<V, E>
+where
+    E: Evaluator<Space = V>,
+    V: Clone + PartialEq + Send + Sync,
+{
+}
 
 /// Like [`LiftedFunction`], but for lazy evaluation with a [`LazyEvaluator`].
 ///
@@ -508,6 +504,21 @@ where
         LiftedLazyFunction(self.0.clone(), self.1.clone(), self.2.clone())
     }
 }
+impl<V, E> PartialEq for LiftedLazyFunction<V, E>
+where
+    E: LazyEvaluator<Space = V>,
+    V: Clone + PartialEq + Send + Sync,
+{
+    fn eq(&self, other: &LiftedLazyFunction<V, E>) -> bool {
+        self.0 == other.0 && self.2 == other.2
+    }
+}
+impl<V, E> Eq for LiftedLazyFunction<V, E>
+where
+    E: LazyEvaluator<Space = V>,
+    V: Clone + PartialEq + Send + Sync,
+{
+}
 
 use self::ReducedExpression::*;
 #[derive(Clone, PartialEq)]
@@ -555,9 +566,8 @@ where
             let next = evaluated.eval(evaluator, env)?;
             if next == evaluated {
                 break;
-            } else {
-                evaluated = next
             }
+            evaluated = next;
         }
         match evaluated {
             Value(o) => Ok(o),
@@ -579,9 +589,8 @@ where
             let next = evaluated.lazy_eval(evaluator, env)?;
             if next == evaluated {
                 break;
-            } else {
-                evaluated = next
             }
+            evaluated = next;
         }
         match evaluated {
             Value(o) => Ok(o),
